#!/usr/bin/env node

const fs = require('fs');
const glob = require("glob");
const css = require('css');
const clean = require('clean-css');
const program = require('commander');
const package = require('./package.json');

// Setup CLI args
program
    .version(package.version)
    .option('-i --input [input]', 'Input directory with CSS Files to combine, defualt is current directory.')
    .option('-o --output [output]', 'Output CSS file name, default is output.css')
    .option('-m --minify', 'Minify the output CSS')
    .parse(process.argv);

let inputDirectory = './' 
if (program.input) {
    inputDirectory = program.input;
};

let outputFile = 'output.css';
if (program.output) {
    outputFile = program.output
}

// Initialize CSS storage object
let selectorsDict = {};

// Find all CSS files in specified directory
glob(inputDirectory + '/*.css', function (er, files) {
    console.log(files.length + ' Input Files');
    console.log(files);
    for (let filePath of files) {
        readFile(filePath);
    }
    writeToFile();
});

// Read the file and parse CSS rules
function readFile(filename) {
    let data = fs.readFileSync(filename, 'utf8');
    let cleanedCss = new clean().minify(data)
    let cleanedCssString = cleanedCss.styles;

    if (cleanedCss.errors.length !== 0) {
        console.error('Error parsing file: ' + filename);
        console.error(cleanedCss.errors);
    }

    if (cleanedCss.warnings.length !== 0) {
        console.error('Warnings parsing file: ' + filename);
        console.error(cleanedCss.warnings);
    }

    let cssObj = css.parse(cleanedCssString, { silent: true });
    if (cssObj.stylesheet.parsingErrors.length === 0) {
        let rules = cssObj.stylesheet.rules;
        mergeCSS(rules);
    }
    else {
        console.error('Error parsing file: ' + filename);
    }
}

// Add each CSS rule to the global dict
function mergeCSS(cssRules) {
    // Iterate through each CSS rule
    for (let rule of cssRules) {
        if (rule.type === 'rule') {
            let selectorsKey = rule.selectors.toString();

            // If the selector is a duplicate, concat the new declarations
            if (selectorsDict[selectorsKey]) {
                selectorsDict[selectorsKey] = selectorsDict[selectorsKey].concat(rule.declarations);
            }
            // Else add the selector to the dict
            else {
                selectorsDict[selectorsKey] = rule.declarations;
            }
        }
    }
}

// Write the unified CSS dict to an output file
function writeToFile() {
    let cssString = '';

    // Write each selector object as a css string
    for (let selector in selectorsDict) {
        cssString += `\n${selector} {\n` 
        for (let declaration of selectorsDict[selector]) {
            cssString += `\t${declaration.property}: ${declaration.value};\n`
        }
        cssString += '}\n' 
    }

    // If --minify was specified in the CLI, minify the output string
    if (program.minify) {
        let minifiedString = new clean().minify(cssString).styles;
        cssString = minifiedString;
    }

    fs.writeFile(outputFile, cssString, function(err) {
        if(err) {
            return console.log(err);
        }

        console.log('Merged file saved as ' + outputFile);
    }); 
}